#include "confparser.hh"

void ConfParser::ParseParam(FILE* fp, ConfParser::ElemType& elem)
{
    int ch = fgetc(fp);
    if(ch == EOF) return;
    if(ch == '"')
    {
        elem.SField = "";
        for(;;)
        {
            ch = fgetc(fp);
            if(ch == EOF) break;
            if(ch == '"') break;
            if(ch == '\\')
            {
                ch = fgetc(fp);
                if(ch == 'n') ch = '\n';
                else if(ch == 'r') ch = '\r';
                else if(ch == 't') ch = '\t';
                else if(ch == EOF) break;
            }
            elem.SField += ch;
        }
    }
    else if(isdigit(ch))
    {
        elem.IField = 0;
        elem.SField = "";
        while(isdigit(ch) && ch != EOF)
        {
            elem.IField = elem.IField * 10 + (ch - '0');
            elem.SField += ch;
            ch = fgetc(fp);
        }
        if(ch != EOF) ungetc(ch, fp);
    }
    else
    {
        fprintf(stderr, "Unexpected character '%c' in config file - expected value\n", ch);
        // ERROR
    }
}

void ConfParser::ParseElement(FILE* fp, ConfParser::SectionType& section, const std::string& name)
{
    FieldType& field = section.fields[name];
    field.values.resize(field.values.size() + 1);
    FuncType& func = field.values[field.values.size() - 1];
    
    int ch;
    for(;;)
    {
        ch = fgetc(fp);
        if(ch == EOF) break;
        if(ch != '\n' && isspace(ch)) continue;
        if(ch == '=') continue;
        ungetc(ch, fp);
        break;
    }
    if(ch == '\n') return;
    for(;;)
    {
        func.params.resize(func.params.size() + 1);
        ElemType& elem = func.params[func.params.size() - 1];
        
        ParseParam(fp, elem);
        
        for(;;)
        {
            ch = fgetc(fp);
            if(ch == '#')
            {
                do { ch = fgetc(fp); } while(ch != '\n' && ch != EOF);
            }
            if(ch == EOF) break;
            if(ch == '\n' || !isspace(ch)) break;
        }
        if(ch == EOF || ch == '\n') break;
        ungetc(ch, fp);
    }
}

void ConfParser::ParseSection(FILE* fp, const std::string& name)
{
    SectionType& section = sections[name];
    
    for(;;)
    {
        int ch = fgetc(fp);
        if(ch == EOF) break;
        if(ch == '#')
        {
            do { ch = fgetc(fp); } while(ch != '\n' && ch != EOF);
            continue;
        }
        if(isspace(ch)) continue;
        if(ch == '[')
        {
            ungetc(ch, fp);
            return;
        }
        if(isalpha(ch))
        {
            std::string elemname;
            ungetc(ch, fp);
            for(;;)
            {
                ch = fgetc(fp);
                if((ch != '_' && !isalnum(ch)) || ch == EOF) break;
                elemname += ch;
            }
            if(ch == EOF) break;
            ungetc(ch, fp);
            ParseElement(fp, section, elemname);
            continue;
        }
        fprintf(stderr, "Unexpected character '%c' in config file - expected declaration\n", ch);
        // ERROR
    }
}
void ConfParser::Parse(FILE* fp)
{
    for(;;)
    {
        int ch = fgetc(fp);
        if(ch == EOF) break;
        if(ch == '#')
        {
            do { ch = fgetc(fp); } while(ch != '\n' && ch != EOF);
            continue;
        }
        if(isspace(ch)) continue;
        if(ch == '[')
        {
            std::string secname;
            for(;;)
            {
                ch = fgetc(fp);
                if(ch == ']' || ch == EOF) break;
                secname += ch;
            }
            if(ch == EOF) break;
            ParseSection(fp, secname);
            continue;
        }
        fprintf(stderr, "Unexpected character '%c' in config file - expected section\n", ch);
        // ERROR
    }
}

const ConfParser::FieldType& ConfParser::FindField
   (const ConfParser::SectionNameType& sect,
    const ConfParser::FieldNameType& field) const
{
    std::map<SectionNameType, SectionType>::const_iterator i;
    std::map<FieldNameType, FieldType>::const_iterator j;
    
    i = sections.find(sect);
    if(i == sections.end()) throw 1;
    
    const SectionType& s = i->second;
    j = s.fields.find(field);
    if(j == s.fields.end()) throw 2;
    
    return j->second;
}
