#ifndef CONFPARSER_HH_
#define CONFPARSER_HH_

#include <vector>
#include <string>
#include <map>

using std::FILE;

class ConfParser
{
public:
    void Parse(FILE *fp);
    void Clear() { sections.clear(); }

    typedef std::string SectionNameType;
    typedef std::string FieldNameType;
 
public:
    class ElemType
    {
    public:
        std::string SField;
        unsigned    IField;
    };
    class FuncType
    {
    private:
        std::vector<ElemType> params;
        friend class ConfParser;
    public:
        const std::string SField() const { return params[0].SField; }
        const unsigned    IField() const { return params[0].IField; }
        
        unsigned size() const { return params.size(); }
        const ElemType& operator[] (unsigned n) const { return params[n]; }
    };
    class FieldType
    {
    private:
        std::vector<FuncType> values;
        friend class ConfParser;
    public:
        const std::string SField() const { return values[0].SField(); }
        const unsigned    IField() const { return values[0].IField(); }
        
        unsigned size() const { return values.size(); }
        const FuncType& operator[] (unsigned n) const { return values[n]; }
    };
    
    const FieldType& FindField(const SectionNameType& sect,
                               const FieldNameType& field) const;

private:
    class SectionType
    {
    public:
        std::map<FieldNameType, FieldType> fields;
    };
    std::map<SectionNameType, SectionType> sections;

//-----------------------------------------------------------------------
private:
    void ParseSection(FILE* fp, const std::string& name);
    void ParseElement(FILE* fp, SectionType& section, const std::string& name);
    void ParseParam(FILE* fp, ElemType& elem);
};

#endif
